#!/usr/bin/env node
import BuildTransactions from "./build.js";
import { Command } from "commander";
import { exec } from "child_process";

const FilterModifiedObjects = () => {
  return ['Table' ,'UserDefinedDataType','UserDefinedTableType', 'Synonym']
}

const CleanArr = (branch, e, isData) => {
  let arr = e.split('\t');
  if (arr[0].length == 0) return false; // get rid of blanks cause by new line split
  if (!['A','M','D'].includes(arr[0]) && !isData) return false; // Filter out files that have a status type of Added or Modified (object only)
  if (!isData && FilterModifiedObjects().includes(arr[1].split('/')[0]) && arr[0].includes('M')) return false; //Check valid modified objects  
  return true;
}
const buildFilesArr = (str, branch = null, isData = false) => {
  return str
    .split("\n")
    .filter((e) => {      
      return CleanArr(branch, e, isData );
    })
    .map((e) => {
      let tmp = e.split("\t");
      return {
        type: tmp[0],
        path: tmp[1],
      };
    });
};

const rearrangeDate = (date) => {
  // split the date by the hyphen character
  let parts = date.split("-");
  // get the month, day, and year parts
  let mm = parts[0];
  let dd = parts[1];
  let yyyy = parts[2];
  // join the parts in the desired order with the hyphen character
  let newDate = [yyyy, mm, dd].join("-");
  // return the new date
  return newDate;
}

const execPromise = (cmd) => {
  return new Promise(function(resolve, reject) {
      exec(cmd, function(err, stdout) {
          if (err) {
            console.log('exec error: ' + err);
            return reject(err)};
          resolve(stdout);
      });
  });
}

const objects = async (
  branch,
  targetbranch,
  options = { name: "Deployment_Object_Script_Raw.sql" }
) => {  
  execPromise(`git diff --name-status ${branch} ${targetbranch} -- Function -- StoredProcedure -- Trigger -- UserDefined* -- View -- Table`).then((stdout) => {
    let filesArr = buildFilesArr(stdout, branch);
    const build = new BuildTransactions();      
    if(!options.revert) build.BuildObjectsFile(filesArr, options);
    
    if(options.revert) {
      execPromise(`git checkout -f ${targetbranch}`).then((stdout) => {
        console.log('checkout', stdout);
        execPromise(`git diff --name-status ${targetbranch} ${branch} -- Function -- StoredProcedure -- Trigger -- UserDefined* -- View -- Table`).then((stdout) => {
          let filesArr = buildFilesArr(stdout, targetbranch);
          const buildRevert = new BuildTransactions();
          options.name = options.name.replace('.sql', '_Revert.sql');
          buildRevert.BuildObjectsFile(filesArr, options);  
        })
      })
    }
  })
  
};

const s3objects = async (startdate, enddate , targetbranch) => {
  let branch = targetbranch.split("/");
  execPromise(`aws s3api list-objects-v2 --bucket bizlibrary-deployments --prefix ${branch[branch.length-1]} --query "Contents[? (LastModified>='${rearrangeDate(startdate)}T00:00:00.00Z' && LastModified<='${rearrangeDate(enddate)}T23:59:59.99Z')].Key"`).then((stdout) => {  
    console.log(stdout);  
    process.exit();
  })
}

const data = (
  branch,
  targetbranch,
  options = { name: "Deployment_Data_Script_Raw.sql" }
) => {
  execPromise(`git diff --name-status ${branch} ${targetbranch} -- Data* -- Redshift* -- Object*`).then((stdout) => {
    let filesArr = buildFilesArr(stdout, branch, true);
    const build = new BuildTransactions();      
    build.BuildDataFile(filesArr, options);    
  })
};

const handleAll = async (branch, targetbranch, options) => {  
  execPromise(`deploysql data ${branch} ${targetbranch}`).then((stdout) => {
    console.log('data', stdout);    
    execPromise(`deploysql data ${branch} ${targetbranch} -r true`).then((stdout) => {
      console.log('data redshift', stdout);
      execPromise(`deploysql objects ${branch} ${targetbranch}`).then((stdout) => {
        console.log('objects', stdout);
        execPromise(`deploysql objects ${branch} ${targetbranch} -r true`).then((stdout) => {
          console.log('objects revert', stdout);
          process.exit();
        })      
      })      
    })
  })
}

const cli = new Command();

cli.description("Tool to build SQL Object transactions from SQL Repo");
cli.name("deploysql");
cli.usage("<command>");

cli
  .command("objects")
  .argument("[branch]", "Name of Branch")
  .argument("[targetbranch]", "Name of Target Branch")
  .option("-r, --revert <boolean>", "Create a revert Build")
  .option("-s, --server <string>", "SQL Connection server")
  .option("-u, --username <string>", "SQL Connection username")
  .option("-p, --password <string>", "SQL Connection password")
  .option("-d, --database <string>", "SQL Connection database")
  .option("-o, --port <int>", "SQL Connection port")
  .option(
    "-n, --name <type>",
    "Filename of output file",
    "Deployment_Object_Script_Raw.sql"
  )
  .description("Build a transactions file of object scripts for SQL deployment")
  .action((branch, targetbranch, options) => {
    objects(branch, targetbranch, options);
  });

cli
  .command("data")
  .argument("[branch]", "Name of Branch")
  .argument("[targetbranch]", "Name of Target Branch")
  .option("-r, --redshift <boolean>", "Redshift Data Script")
  .option("-s, --server <string>", "SQL Connection server")
  .option("-u, --username <string>", "SQL Connection username")
  .option("-p, --password <string>", "SQL Connection password")
  .option("-d, --database <string>", "SQL Connection database")
  .option("-o, --port <int>", "SQL Connection port")
  .option(
    "-n, --name <type>",
    "Filename of output file",
    "Deployment_Data_Script_Raw.sql"
  )
  .description("Build a transactions file of data scripts for SQL deployment")
  .action((branch, targetbranch, options) => {
    data(branch, targetbranch, options);
  });

cli
  .command("all")
  .argument("[branch]", "Name of Branch")
  .argument("[targetbranch]", "Name of Target Branch")
  .option("-s, --server <type>", "SQL Connection server")
  .option("-u, --username <type>", "SQL Connection username")
  .option("-p, --password <type>", "SQL Connection password")
  .option("-d, --database <type>", "SQL Connection database")
  .option("-o, --port <type>", "SQL Connection port")
  .description("Build a tranaction file from all scripts for Sql Deployment")
  .action((branch, targetbranch, options) => {
    handleAll(branch, targetbranch, options);
  });

  cli
  .command("s3build")
  .argument("[startdate]", "Start Date MM-DD-YYYY Format")
  .argument("[enddate]", "End Date MM-DD-YYYY Format")  
  .argument("[targetbranch]", "Name of Target Branch")
  .description("Build a tranaction file from s3 for all scripts for Sql Deployment")
  .action((startdate, enddate , targetbranch) => {    
    s3objects(startdate, enddate , targetbranch);
  });
cli.parse(process.argv);
